"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = buildExpressionTemplate;

var _template = _interopRequireDefault(require("@babel/template"));

var _invariant = _interopRequireDefault(require("../invariant.js"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Copyright (c) 2017-present, Facebook, Inc.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree. An additional grant
 * of patent rights can be found in the PATENTS file in the same directory.
 */
const placeholders = "ABCDEFGHIJKLMNOPQRSTUVWXYZ".split("");
const placeholderDefaultWhiteList = new Set(["global"]);
const placeholderWhitelist = new Set([...placeholderDefaultWhiteList, ...placeholders]);

function buildExpressionTemplate(code, {
  disablePlaceholders
} = {}) {
  let template;
  return preludeGenerator => obj => {
    if (template === undefined) template = (0, _template.default)(code, {
      placeholderPattern: false,
      placeholderWhitelist: disablePlaceholders ? placeholderDefaultWhiteList : placeholderWhitelist
    });
    if (preludeGenerator !== undefined && code.includes("global")) obj = Object.assign({
      global: preludeGenerator.memoizeReference("global")
    }, obj);
    let result = template(obj).expression;
    (0, _invariant.default)(result !== undefined, "Code does not represent an expression: " + code);
    return result;
  };
}
//# sourceMappingURL=builder.js.map